from authserv.test import *
from authserv.ratelimit import *


class RateLimitTest(unittest.TestCase):

    def setUp(self):
        self.tick = 0
        def _time():
            return self.tick
        self.mc = FakeMemcache(_time)
        self.rl = RateLimit(100, 10)

    def test_ratelimit_pass(self):
        n = 200
        ok = 0
        for i in xrange(n):
            self.tick = i
            if self.rl.check(self.mc, 'key'):
                ok += 1
        self.assertEquals(n, ok)

    def test_ratelimit_fail(self):
        n = 200
        ok = 0
        for i in xrange(n):
            self.tick = i / 20
            if self.rl.check(self.mc, 'key'):
                ok += 1
        self.assertEquals(100, ok)


class BlackListTest(unittest.TestCase):

    def setUp(self):
        self.tick = 0
        def _time():
            return self.tick
        self.mc = FakeMemcache(_time)
        self.bl = BlackList(100, 10, 3600)

    def test_block(self):
        for i in xrange(100):
            self.bl.incr(self.mc, 'key')
            self.assertTrue(self.bl.check(self.mc, 'key'))
        # 101th request is blocked
        self.bl.incr(self.mc, 'key')
        self.assertFalse(self.bl.check(self.mc, 'key'))
        # check expiration of the block
        self.tick = 3601
        self.assertTrue(self.bl.check(self.mc, 'key'))
